from typing import Any

# isort: split
from ansys.meshing.prime.autogen.part import Part as _Part

# isort: split
from ansys.meshing.prime.autogen.commonstructs import SetNameResults as SetNameResults
from ansys.meshing.prime.autogen.partstructs import PartSummaryParams


class Part(_Part):
    __doc__ = _Part.__doc__

    def __init__(self, model, id: int, object_id: int, name: str):
        """Initialize Part

        Parameters
        ----------
        model: ansys.meshing.prime.Model
            Model in which part is created
        id: int
            Id of the part provided by server
        object_id: int
            Object id provided by the server
        name: str
            Part name
        """
        self._model = model
        self._print_mesh = False
        self._print_id = False
        _Part.__init__(self, model, id, object_id, name)

    def __call__(self, *args: Any, **kwds: Any) -> str:
        """Callable interface of the Part.

        Gets summary of the part using supported keyword arguments as given below.

        Parameters
        ----------
        print_mesh : bool, optional
            Passing True will get the mesh summary along with part summary.
            The default is False.
        peint_id : bool, optional
            Passing True will get id's of topo entities/zonelets along with part summary.
            The default is False.

        Returns
        -------
        str
            Returns the summary of part.

        Examples
        --------
        >>> from ansys.meshing.prime import Model
        >>> model = client.model
        >>> part = model.get_part_by_name("Part.1")
        >>> print(part(print_mesh=True, print_id=True))
        """
        params = PartSummaryParams(
            model=self._model, print_id=self._print_id, print_mesh=self._print_mesh
        )
        for key, value in kwds.items():
            setattr(params, key, value)
            if key == 'print_mesh':
                params.print_mesh = value
            if key == 'print_id':
                params.print_id = value
        result = _Part.get_summary(self, params)
        return result.message

    def __str__(self) -> str:
        """Prints the summary of a part.

        Uses print_mesh and print_id properties to control the the summary of a part.

        Returns
        -------
        str
            Returns the summary of a part.

        Examples
        --------
        >>> from ansys.meshing.prime import Model
        >>> model = client.model
        >>> part = model.get_part_by_name("Part.1")
        >>> print(part)
        """
        params = PartSummaryParams(model=self._model)
        params.print_mesh = self._print_mesh
        params.print_id = self._print_id
        result = _Part.get_summary(self, params)
        return result.message

    def set_suggested_name(self, name: str) -> SetNameResults:
        """Sets the unique name for the part based on the given suggested name.


        Parameters
        ----------
        name : str
            Suggested name for the part.

        Returns
        -------
        SetNameResults
            Returns the results with assigned name of the part.


        Examples
        --------
        >>> part.set_suggested_name("part1")

        """
        result = _Part.set_suggested_name(self, name)
        self._name = result.assigned_name
        return result

    @property
    def print_mesh(self) -> bool:
        """When True, prints the mesh summary along with part summary. The default is False."""
        return self._print_mesh

    @print_mesh.setter
    def print_mesh(self, value: bool):
        self._print_mesh = value

    @property
    def print_id(self) -> bool:
        """When True, prints the id's of topoentities or zonelets along with part summary.
        The default is False.
        """
        return self._print_id

    @print_id.setter
    def print_id(self, value: bool):
        self._print_id = value



import logging
import os
import shutil
import subprocess
from contextlib import contextmanager
from typing import Optional

import ansys.meshing.prime.internals.config as config
import ansys.meshing.prime.internals.defaults as defaults

_LOCAL_PORTS = []


def to_camel_case(snake_str):
    components = snake_str.split('_')
    # We capitalize the first letter of each component except the first one
    # with the 'title' method and join them together.
    return components[0] + ''.join(x.title() for x in components[1:])


def get_child_processes(process):
    children = []
    cmd = subprocess.Popen("pgrep -P %d" % process, shell=True, stdout=subprocess.PIPE)
    out = cmd.stdout.read().decode("utf-8")
    cmd.wait()
    for pid in out.split("\n")[:1]:
        if pid.strip() == '':
            break
        ps_cmd = subprocess.Popen(
            "ps -o cmd= {}".format(int(pid)), stdout=subprocess.PIPE, shell=True
        )
        ps_out = ps_cmd.stdout.read().decode("utf-8")
        ps_cmd.wait()
        cmd_name = ps_out.split()[0]
        if "AnsysPrimeServer" in cmd_name:
            children.append(int(pid))
        else:
            children += get_child_processes(int(pid))
    return children


def terminate_process(process):
    import signal
    import sys

    if sys.platform.startswith('win32'):
        # process.send_signal(signal.CTRL_C_EVENT)
        process.send_signal(signal.CTRL_BREAK_EVENT)
    else:
        for child in get_child_processes(process.pid):
            os.kill(child, signal.SIGTERM)

    if process.stdin is not None:
        process.stdin.close()
    if process.stdout is not None:
        process.stdout.close()
    if process.stderr is not None:
        process.stderr.close()
    process.terminate()
    process.wait()


def print_logs_before_command(logger: logging.Logger, command: str, args):
    logger.info("Executing " + command)
    if logger.isEnabledFor(logging.DEBUG):
        logger.debug("Command: " + command)
        logger.debug("Args:")
        for key in args:
            logger.debug("    " + key + ":")
            val = args[key]
            printable_str = ""
            if hasattr(val, '__str__'):
                printable_str = val.__str__()
            elif type(val) == 'str':
                printable_str = val
            else:
                printable_str = str(val)
            for line in printable_str.splitlines():
                logger.debug("        " + line)
        logger.debug("")


def print_logs_after_command(logger: logging.Logger, command: str, ret):
    logger.info("Finished " + command)
    if logger.isEnabledFor(logging.DEBUG):
        logger.debug("Return: ")
        printable_str = ""
        if hasattr(ret, '__str__'):
            printable_str = ret.__str__()
        elif type(ret) == 'str':
            printable_str = ret
        else:
            printable_str = str(ret)
        for line in printable_str.splitlines():
            logger.debug("        " + line)
        logger.debug("")


def launch_prime_github_container(
    mount_host: str = defaults.get_user_data_path(),
    mount_image: str = defaults.get_user_data_path_for_containers(),
    port: int = defaults.port(),
    name: str = 'ansys-prime-server',
    version: Optional[str] = None,
):
    license_file = os.environ.get('ANSYSLMD_LICENSE_FILE', None)
    image_name = os.environ.get('PYPRIMEMESH_IMAGE_NAME', 'ghcr.io/pyansys/prime')
    if license_file is None:
        raise ValueError('Licensing information to launch container not found')
    if version is None:
        version = os.environ.get('PYPRIMEMESH_IMAGE_TAG', 'latest')
    docker_command = [
        'docker',
        'run',
        '-d',
        '--rm',
        '--name',
        f'{name}',
        '-p',
        f'{port}:{port}',
        '-v',
        f'{mount_host}:{mount_image}',
        '-e',
        f'ANSYSLMD_LICENSE_FILE={license_file}',
        f'{image_name}:{version}',
        '--port',
        f'{port}',
    ]
    subprocess.run(docker_command, stdout=subprocess.DEVNULL)


def stop_prime_github_container(name):
    subprocess.run(['docker', 'stop', f'{name}'], stdout=subprocess.DEVNULL)


@contextmanager
def file_read_context(model, file_name):
    if config.using_container():
        base_file_name = os.path.basename(file_name)
        temp_file_name = os.path.join(defaults.get_examples_path(), base_file_name)
        is_copy: bool = file_name != temp_file_name
        if is_copy:
            shutil.copyfile(file_name, temp_file_name)
        container_file_name = os.path.join(
            defaults.get_examples_path_for_containers(), base_file_name
        )
        container_file_name = container_file_name.replace(os.path.sep, '/')
        yield container_file_name
        if is_copy:
            os.remove(temp_file_name)
    elif config.has_pim():
        temp_file_name = os.path.basename(file_name)
        model.file_service.upload_file(file_name)
        yield temp_file_name
    else:
        yield file_name


def port_in_use(port, host=defaults.ip()):
    """Returns True when a port is in use at the given host.
    Must actually "bind" the address.  Just checking if we can create
    a socket is insufficient as it's possible to run into permission
    errors like:
    - An attempt was made to access a socket in a way forbidden by its
      access permissions.
    """
    import socket

    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
        try:
            sock.bind((host, port))
            return False
        except:
            return True


def get_available_local_port(init_port: int = defaults.port()):
    port = init_port
    while port_in_use(port) or port in _LOCAL_PORTS:
        port += 1
    _LOCAL_PORTS.append(port)
    return port


@contextmanager
def file_read_context_list(model, file_names):
    if config.using_container():
        base_names = [os.path.basename(file) for file in file_names]
        temp_names = [os.path.join(defaults.get_examples_path(), base) for base in base_names]
        for file, temp in zip(file_names, temp_names):
            shutil.copyfile(file, temp)
        container_files = [
            os.path.join(defaults.get_examples_path_for_containers(), base) for base in base_names
        ]
        container_files = [file.replace(os.path.sep, '/') for file in container_files]
        yield container_files
        for temp_file in temp_names:
            os.remove(temp_file)
    elif config.has_pim():
        temp_files = [os.path.basename(file) for file in file_names]
        for file in file_names:
            model.file_service.upload_file(file)
        yield temp_files
    else:
        yield file_names


@contextmanager
def file_write_context(model, file_name):
    if config.using_container():
        base_file_name = os.path.basename(file_name)
        temp_file_name = os.path.join(defaults.get_output_path_for_containers(), base_file_name)
        temp_file_name = temp_file_name.replace(os.path.sep, '/')
        if not os.path.exists(defaults.get_output_path()):
            os.makedirs(defaults.get_output_path())
        yield temp_file_name
        # Copy temp_file_name to directory which was asked
        local_file_name = os.path.join(defaults.get_output_path(), base_file_name)
        shutil.copyfile(local_file_name, file_name)
        os.remove(local_file_name)
    elif config.has_pim():
        temp_file_name = os.path.basename(file_name)
        file_dir = os.path.dirname(file_name)
        yield temp_file_name
        model.file_service.download_file(temp_file_name, file_dir)
    else:
        yield file_name



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class MaterialPointManager(CoreObject):
    """Provides functions for material point creation, deletion and queries.

    """

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """ Initialize MaterialPointManager """
        self._model = model
        self._comm = model._communicator
        self._id = id
        self._object_id = object_id
        self._name = name
        self._freeze()

    def create_material_point(self, suggested_name : str, coords : Iterable[float], params : CreateMaterialPointParams) -> CreateMaterialPointResults:
        """ Creates a material point at the given coordinates.


        Parameters
        ----------
        suggested_name : str
            A name suggestion for material point.
        coords : Iterable[float]
            Coordinates of material point.
        params : CreateMaterialPointParams
            Parameters to material point.

        Returns
        -------
        CreateMaterialPointResults
            Returns the result with material point name and id.

        Notes
        -----
        A material point is created on calling this API.

        Examples
        --------
        >>> material_point_results = model.control_data.create_material_point("Fluid", [1.0,2,0,3.0], params)

        """
        args = {"suggested_name" : suggested_name,
        "coords" : coords,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::MaterialPointManager/CreateMaterialPoint"
        self._model._print_logs_before_command("create_material_point", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("create_material_point", CreateMaterialPointResults(model = self._model, json_data = result))
        return CreateMaterialPointResults(model = self._model, json_data = result)

    def delete_material_point(self, name : str) -> DeleteMaterialPointResults:
        """ Deletes material point identified with the given name.


        Parameters
        ----------
        name : str
            Name of the material point to be deleted.

        Returns
        -------
        DeleteMaterialPointResults
            Returns the DeleteMaterialPointResults.


        Examples
        --------
        >>> model = prime.local_model()
        >>> results = model.material_point_data.delete_material_point("fluid")

        """
        args = {"name" : name}
        command_name = "PrimeMesh::MaterialPointManager/DeleteMaterialPoint"
        self._model._print_logs_before_command("delete_material_point", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("delete_material_point", DeleteMaterialPointResults(model = self._model, json_data = result))
        return DeleteMaterialPointResults(model = self._model, json_data = result)

    @property
    def id(self):
        """ Get the id of MaterialPointManager."""
        return self._id

    @property
    def name(self):
        """ Get the name of MaterialPointManager."""
        return self._name



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class VolumeControl(CoreObject):
    """Volume controls provide volume specific settings on volumes specified by scope and settings specified by parameters.

    """

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """ Initialize VolumeControl """
        self._model = model
        self._comm = model._communicator
        self._id = id
        self._object_id = object_id
        self._name = name
        self._freeze()

    def set_params(self, volume_control_params : VolumeControlParams):
        """ Sets the volume control parameters.


        Parameters
        ----------
        volume_control_params : VolumeControlParams
            Parameters to control volume.

        Examples
        --------
        >>> volume_control.set_params(
        >>>                  prime.VolumeControlParams(model=model,
        >>>                  cell_zonelet_type = prime.CellZoneletType.FLUID))

        """
        args = {"volume_control_params" : volume_control_params._jsonify()}
        command_name = "PrimeMesh::VolumeControl/SetParams"
        self._model._print_logs_before_command("set_params", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_params")

    def set_scope(self, scope : ScopeDefinition) -> SetScopeResults:
        """ Sets the scope for volume control to evaluate.

        Volume control uses scope to evaluate entities for which volume mesh needs to be generated.

        Parameters
        ----------
        scope : ScopeDefinition
            ScopeDefinition to scope entities for volume mesh generation.

        Returns
        -------
        SetScopeResults
            Returns a SetScopeResults.


        Examples
        --------
        >>> volume_control.set_scope(prime.ScopeDefinition(model=model,
        >>>                        entity_type = ScopeEntity.VOLUMES,
        >>>                        evaluation_type = ScopeEvaluationType.ZONES,
        >>>                        zone_expression = "vol_in"))

        """
        args = {"scope" : scope._jsonify()}
        command_name = "PrimeMesh::VolumeControl/SetScope"
        self._model._print_logs_before_command("set_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_scope", SetScopeResults(model = self._model, json_data = result))
        return SetScopeResults(model = self._model, json_data = result)

    def set_suggested_name(self, name : str) -> SetNameResults:
        """ Sets the unique name for the volume control based on the given suggested name.


        Parameters
        ----------
        name : str
            Suggested name for the volume control.

        Returns
        -------
        SetNameResults
            Returns a name of the volume control.


        Examples
        --------
        >>> volume_control.set_suggested_name("control1")

        """
        args = {"name" : name}
        command_name = "PrimeMesh::VolumeControl/SetSuggestedName"
        self._model._print_logs_before_command("set_suggested_name", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_suggested_name", SetNameResults(model = self._model, json_data = result))
        return SetNameResults(model = self._model, json_data = result)

    def get_scope(self) -> ScopeDefinition:
        """ Gets the scope for the volume control.


        Returns
        -------
        ScopeDefinition
            Returns scope of the volume control.


        Examples
        --------
        >>> scope_definition = volume_control.get_scope()

        """
        args = {}
        command_name = "PrimeMesh::VolumeControl/GetScope"
        self._model._print_logs_before_command("get_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_scope", ScopeDefinition(model = self._model, json_data = result))
        return ScopeDefinition(model = self._model, json_data = result)

    def get_params(self) -> VolumeControlParams:
        """ Get the parameters of the volume control.


        Returns
        -------
        VolumeControlParams
            Return parameters of the volume control.


        Examples
        --------
        >>> params = volume_control.get_params()

        """
        args = {}
        command_name = "PrimeMesh::VolumeControl/GetParams"
        self._model._print_logs_before_command("get_params", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_params", VolumeControlParams(model = self._model, json_data = result))
        return VolumeControlParams(model = self._model, json_data = result)

    @property
    def id(self):
        """ Get the id of VolumeControl."""
        return self._id

    @property
    def name(self):
        """ Get the name of VolumeControl."""
        return self._name



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class Part(CoreObject):
    """Part contains zonelets and topoentities.

    Topoentities and zonelets are characterized by dimension of entities.
    Zonelets are a group of interconnected elements in a mesh. There are three types of zonelets. They are:

    * FaceZonelet: A group of interconnected face elements.
    * EdgeZonelet: A group of interconnected edge elements.
    * CellZonelet: A group of interconnected cell elements.

    Topoentities represent connectivity information.
    Topoentities can be queried from higher order to lower order topoentities and vice versa.
    Topoentities have geometric representation which may be defined by splines or facets.
    The mesh generated on topoentities will be projected on geometry representation.

    * TopoFace: Topoentity representing surfaces.
    * TopoEdge: Topoentity representing curves.
    * TopoVolume: Topoentity representing volumes.
    """

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """ Initialize Part """
        self._model = model
        self._comm = model._communicator
        self._id = id
        self._object_id = object_id
        self._name = name
        self._freeze()

    def get_name(self) -> str:
        """ Gets the name of the Part.


        Returns
        -------
        str
            Returns part name.


        Examples
        --------
        >>> part_name = part.get_name()

        """
        args = {}
        command_name = "PrimeMesh::Part/GetName"
        self._model._print_logs_before_command("get_name", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_name")
        return result

    def set_suggested_name(self, name : str) -> SetNameResults:
        """ Sets the unique name for the part based on the suggested name.


        Parameters
        ----------
        name : str
            Suggested name for the part.

        Returns
        -------
        SetNameResults
            Returns the SetNameResults.


        Examples
        --------
        >>> part.set_suggested_name("part1")

        """
        args = {"name" : name}
        command_name = "PrimeMesh::Part/SetSuggestedName"
        self._model._print_logs_before_command("set_suggested_name", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_suggested_name", SetNameResults(model = self._model, json_data = result))
        return SetNameResults(model = self._model, json_data = result)

    def get_face_zonelets(self) -> Iterable[int]:
        """ Get the face zonelets of a part.


        Returns
        -------
        Iterable[int]
            Return the ids of face zonelets.


        Examples
        --------
        >>> face_zonelets = part.get_face_zonelets()

        """
        args = {}
        command_name = "PrimeMesh::Part/GetFaceZonelets"
        self._model._print_logs_before_command("get_face_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_face_zonelets")
        return result

    def get_cell_zonelets(self) -> Iterable[int]:
        """ Get the cell zonelet ids in the part.


        Returns
        -------
        Iterable[int]
            Return the ids of cell zonelets.


        Examples
        --------
        >>> from ansys.meshing.prime import Part
        >>> cell_zonelet_ids = part.get_cell_zonelets()

        """
        args = {}
        command_name = "PrimeMesh::Part/GetCellZonelets"
        self._model._print_logs_before_command("get_cell_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_cell_zonelets")
        return result

    def get_edge_zonelets(self) -> Iterable[int]:
        """ Get the edge zonelets of a part.


        Returns
        -------
        Iterable[int]
            Return the ids of edge zonelets.


        Examples
        --------
        >>> edge_zonelets = part.get_edge_zonelets()

        """
        args = {}
        command_name = "PrimeMesh::Part/GetEdgeZonelets"
        self._model._print_logs_before_command("get_edge_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_edge_zonelets")
        return result

    def add_labels_on_zonelets(self, labels : List[str], zonelets : Iterable[int]) -> AddLabelResults:
        """ Add the given labels on the provided zonelets.


        Parameters
        ----------
        labels : List[str]
            Labels to be added on zonelets.
        zonelets : Iterable[int]
            Ids of zonelets.

        Returns
        -------
        AddLabelResults
            Returns the AddLabelResults.


        Examples
        --------
        >>> labels = ["wall", "outer"]
        >>> part.add_labels_on_zonelets(labels, zonelets)

        """
        args = {"labels" : labels,
        "zonelets" : zonelets}
        command_name = "PrimeMesh::Part/AddLabelsOnZonelets"
        self._model._print_logs_before_command("add_labels_on_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("add_labels_on_zonelets", AddLabelResults(model = self._model, json_data = result))
        return AddLabelResults(model = self._model, json_data = result)

    def remove_labels_from_zonelets(self, labels : List[str], zonelets : Iterable[int]) -> RemoveLabelResults:
        """ Remove the given labels from the provided zonelets.


        Parameters
        ----------
        labels : List[str]
            Labels to be removed from zonelets.
        zonelets : Iterable[int]
            Ids of zonelets.

        Returns
        -------
        RemoveLabelResults
            Returns the RemoveLabelResults.


        Examples
        --------
        >>> labels = ["wall", "outer"]
        >>> part.remove_labels_from_zonelets(labels, zonelets)

        """
        args = {"labels" : labels,
        "zonelets" : zonelets}
        command_name = "PrimeMesh::Part/RemoveLabelsFromZonelets"
        self._model._print_logs_before_command("remove_labels_from_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("remove_labels_from_zonelets", RemoveLabelResults(model = self._model, json_data = result))
        return RemoveLabelResults(model = self._model, json_data = result)

    def add_labels_on_topo_entities(self, labels : List[str], topo_entities : Iterable[int]) -> AddLabelResults:
        """ Add the given labels on the provided topoentities.


        Parameters
        ----------
        labels : List[str]
            Labels to be added on topoentities.
        topo_entities : Iterable[int]
            Ids of topoentities.

        Returns
        -------
        AddLabelResults
            Returns the AddLabelResults.


        Examples
        --------
        >>> labels = ["wall", "outer"]
        >>> part.add_labels_on_topo_entities(labels, topo_entities)

        """
        args = {"labels" : labels,
        "topo_entities" : topo_entities}
        command_name = "PrimeMesh::Part/AddLabelsOnTopoEntities"
        self._model._print_logs_before_command("add_labels_on_topo_entities", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("add_labels_on_topo_entities", AddLabelResults(model = self._model, json_data = result))
        return AddLabelResults(model = self._model, json_data = result)

    def remove_labels_from_topo_entities(self, labels : List[str], topo_entities : Iterable[int]) -> RemoveLabelResults:
        """ Remove the given labels from the provided topoentities.


        Parameters
        ----------
        labels : List[str]
            Labels to be removed from topoentities.
        topo_entities : Iterable[int]
            Ids of topoentities.

        Returns
        -------
        RemoveLabelResults
            Returns the RemoveLabelResults.


        Examples
        --------
        >>> labels = ["wall", "outer"]
        >>> part.remove_labels_from_topo_entities(labels, topo_entities)

        """
        args = {"labels" : labels,
        "topo_entities" : topo_entities}
        command_name = "PrimeMesh::Part/RemoveLabelsFromTopoEntities"
        self._model._print_logs_before_command("remove_labels_from_topo_entities", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("remove_labels_from_topo_entities", RemoveLabelResults(model = self._model, json_data = result))
        return RemoveLabelResults(model = self._model, json_data = result)

    def get_face_zones_of_name_pattern(self, zone_name_pattern : str, name_pattern_params : NamePatternParams) -> Iterable[int]:
        """ Get ids of face zones with name matching the given name pattern.


        Parameters
        ----------
        zone_name_pattern : str
            Name pattern to be matched with zone name.
        name_pattern_params : NamePatternParams
            Name pattern parameters used to match zone name pattern.

        Returns
        -------
        Iterable[int]
            Return list of face zone ids matching the zone name pattern.


        Examples
        --------
        >>> name_pattern_params = prime.NamePatternParams(model = model)
        >>> zones = part.get_face_zones_of_name_pattern("wall*", name_pattern_params)

        """
        args = {"zone_name_pattern" : zone_name_pattern,
        "name_pattern_params" : name_pattern_params._jsonify()}
        command_name = "PrimeMesh::Part/GetFaceZonesOfNamePattern"
        self._model._print_logs_before_command("get_face_zones_of_name_pattern", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_face_zones_of_name_pattern")
        return result

    def get_volume_zones_of_name_pattern(self, zone_name_pattern : str, name_pattern_params : NamePatternParams) -> Iterable[int]:
        """ Get ids of volume zones with name matching the given name pattern.


        Parameters
        ----------
        zone_name_pattern : str
            Name pattern to be matched with zone name.
        name_pattern_params : NamePatternParams
            Name pattern parameters used to match zone name pattern.

        Returns
        -------
        Iterable[int]
            Returns a list of volume zone ids matching the zone name pattern.


        Examples
        --------
        >>> name_pattern_params = prime.NamePatternParams(model = model)
        >>> zones = part.get_volume_zones_of_name_pattern("solid*", name_pattern_params)

        """
        args = {"zone_name_pattern" : zone_name_pattern,
        "name_pattern_params" : name_pattern_params._jsonify()}
        command_name = "PrimeMesh::Part/GetVolumeZonesOfNamePattern"
        self._model._print_logs_before_command("get_volume_zones_of_name_pattern", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_volume_zones_of_name_pattern")
        return result

    def get_face_zonelets_of_zone_name_pattern(self, zone_name_pattern : str, name_pattern_params : NamePatternParams) -> Iterable[int]:
        """ Get ids of face zonelets of zones with name matching the given name pattern.


        Parameters
        ----------
        zone_name_pattern : str
            Name pattern to be matched with zone name.
        name_pattern_params : NamePatternParams
            Name pattern parameters used to match zone name pattern.

        Returns
        -------
        Iterable[int]
            Return face zonelet ids of zones with name matching the name pattern.


        Examples
        --------
        >>> name_pattern_params = prime.NamePatternParams(model = model)
        >>> face_zonelets = part.get_face_zonelets_of_zone_name_pattern("wall*", name_pattern_params)

        """
        args = {"zone_name_pattern" : zone_name_pattern,
        "name_pattern_params" : name_pattern_params._jsonify()}
        command_name = "PrimeMesh::Part/GetFaceZoneletsOfZoneNamePattern"
        self._model._print_logs_before_command("get_face_zonelets_of_zone_name_pattern", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_face_zonelets_of_zone_name_pattern")
        return result

    def get_volumes_of_zone_name_pattern(self, zone_name_pattern : str, name_pattern_params : NamePatternParams) -> Iterable[int]:
        """ Get volume ids of zones with name matching the given name pattern.


        Parameters
        ----------
        zone_name_pattern : str
            Name pattern to be matched with zone name.
        name_pattern_params : NamePatternParams
            Name pattern parameters used to match zone name pattern.

        Returns
        -------
        Iterable[int]
            Return volume ids of zones with name matching the name pattern.


        Examples
        --------
        >>> name_pattern_params = prime.NamePatternParams(model = model)
        >>> volumes = part.get_volumes_of_zone_name_pattern("body*", name_pattern_params)

        """
        args = {"zone_name_pattern" : zone_name_pattern,
        "name_pattern_params" : name_pattern_params._jsonify()}
        command_name = "PrimeMesh::Part/GetVolumesOfZoneNamePattern"
        self._model._print_logs_before_command("get_volumes_of_zone_name_pattern", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_volumes_of_zone_name_pattern")
        return result

    def get_topo_faces_of_zone_name_pattern(self, zone_name_pattern : str, name_pattern_params : NamePatternParams) -> Iterable[int]:
        """ Get topoface ids of zones with name matching the given name pattern.


        Parameters
        ----------
        zone_name_pattern : str
            Name pattern to be matched with zone name.
        name_pattern_params : NamePatternParams
            Name pattern parameters used to match zone name pattern.

        Returns
        -------
        Iterable[int]
            Return topoface ids of zones with name matching the name pattern.


        Examples
        --------
        >>> name_pattern_params = prime.NamePatternParams(model = model)
        >>> topo_faces = part.get_topo_faces_of_zone_name_pattern("wall*", name_pattern_params)

        """
        args = {"zone_name_pattern" : zone_name_pattern,
        "name_pattern_params" : name_pattern_params._jsonify()}
        command_name = "PrimeMesh::Part/GetTopoFacesOfZoneNamePattern"
        self._model._print_logs_before_command("get_topo_faces_of_zone_name_pattern", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_topo_faces_of_zone_name_pattern")
        return result

    def get_edge_zonelets_of_label_name_pattern(self, label_name_pattern : str, name_pattern_params : NamePatternParams) -> Iterable[int]:
        """ Get edge zonelet ids of labels with name matching the given name pattern.


        Parameters
        ----------
        label_name_pattern : str
            Name pattern to be matched with label name.
        name_pattern_params : NamePatternParams
            Name pattern parameters used to match label name pattern.

        Returns
        -------
        Iterable[int]
            Return edge zonelet ids of labels with name matching the name pattern.


        Examples
        --------
        >>> name_pattern_params = prime.NamePatternParams(model = model)
        >>> edge_zonelets = part.get_edge_zonelets_of_label_name_pattern("wall*", name_pattern_params)

        """
        args = {"label_name_pattern" : label_name_pattern,
        "name_pattern_params" : name_pattern_params._jsonify()}
        command_name = "PrimeMesh::Part/GetEdgeZoneletsOfLabelNamePattern"
        self._model._print_logs_before_command("get_edge_zonelets_of_label_name_pattern", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_edge_zonelets_of_label_name_pattern")
        return result

    def get_face_zonelets_of_label_name_pattern(self, label_name_pattern : str, name_pattern_params : NamePatternParams) -> Iterable[int]:
        """ Get face zonelet ids of labels with name matching the given name pattern.


        Parameters
        ----------
        label_name_pattern : str
            Name pattern to be matched with label name.
        name_pattern_params : NamePatternParams
            Name pattern parameters used to match label name pattern.

        Returns
        -------
        Iterable[int]
            Return face zonelet ids of labels with name matching the name pattern.


        Examples
        --------
        >>> name_pattern_params = prime.NamePatternParams(model = model)
        >>> face_zonelets = part.get_face_zonelets_of_label_name_pattern("wall*", name_pattern_params)

        """
        args = {"label_name_pattern" : label_name_pattern,
        "name_pattern_params" : name_pattern_params._jsonify()}
        command_name = "PrimeMesh::Part/GetFaceZoneletsOfLabelNamePattern"
        self._model._print_logs_before_command("get_face_zonelets_of_label_name_pattern", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_face_zonelets_of_label_name_pattern")
        return result

    def get_topo_edges_of_label_name_pattern(self, label_name_pattern : str, name_pattern_params : NamePatternParams) -> Iterable[int]:
        """ Get topoedge ids of labels with name matching the given name pattern.


        Parameters
        ----------
        label_name_pattern : str
            Name pattern to be matched with label name.
        name_pattern_params : NamePatternParams
            Name pattern parameters used to match label name pattern.

        Returns
        -------
        Iterable[int]
            Return the ids of topoedges.


        Examples
        --------
        >>> topo_edges = part.get_topo_edges_of_label_name_pattern(
        >>>                   label_name_pattern = "edge_label",
        >>>                   params = prime.NamePatternParams(model=model))

        """
        args = {"label_name_pattern" : label_name_pattern,
        "name_pattern_params" : name_pattern_params._jsonify()}
        command_name = "PrimeMesh::Part/GetTopoEdgesOfLabelNamePattern"
        self._model._print_logs_before_command("get_topo_edges_of_label_name_pattern", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_topo_edges_of_label_name_pattern")
        return result

    def get_topo_faces_of_label_name_pattern(self, label_name_pattern : str, name_pattern_params : NamePatternParams) -> Iterable[int]:
        """ Get topoface ids of labels with name matching the given name pattern.


        Parameters
        ----------
        label_name_pattern : str
            Name pattern to be matched with label name.
        name_pattern_params : NamePatternParams
            Name pattern parameters used to match label name pattern.

        Returns
        -------
        Iterable[int]
            Return the ids of topofaces.


        Examples
        --------
        >>> topo_faces = part.get_topo_faces_of_label_name_pattern(
        >>>                   label_name_pattern = "face_label",
        >>>                   params = prime.NamePatternParams(model=model))

        """
        args = {"label_name_pattern" : label_name_pattern,
        "name_pattern_params" : name_pattern_params._jsonify()}
        command_name = "PrimeMesh::Part/GetTopoFacesOfLabelNamePattern"
        self._model._print_logs_before_command("get_topo_faces_of_label_name_pattern", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_topo_faces_of_label_name_pattern")
        return result

    def merge_zonelets(self, zonelets : Iterable[int], params : MergeZoneletsParams) -> MergeZoneletsResults:
        """ Merge zonelets.


        Parameters
        ----------
        zonelets : Iterable[int]
            Ids of zonelets to be merged.
        params : MergeZoneletsParams
            Parameters to merge zonelets.

        Returns
        -------
        MergeZoneletsResults
            Returns the MergeZoneletsResults.


        Examples
        --------
        params = prime.MergeZoneletsParams(model = model)
        results = part.merge_zonelets(zonelets, params)

        """
        args = {"zonelets" : zonelets,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Part/MergeZonelets"
        self._model._print_logs_before_command("merge_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("merge_zonelets", MergeZoneletsResults(model = self._model, json_data = result))
        return MergeZoneletsResults(model = self._model, json_data = result)

    def merge_volumes(self, volumes : Iterable[int], params : MergeVolumesParams) -> MergeVolumesResults:
        """ Merge volumes by removing shared face zonelets.


        Parameters
        ----------
        volumes : Iterable[int]
            Ids of volumes to be merged.
        params : MergeVolumesParams
            Parameters to merge volumes.

        Returns
        -------
        MergeVolumesResults
            Returns the MergeVolumesResults.


        Examples
        --------
        params = prime.MergeVolumesParams(model = model)
        results = part.merge_volumes(volumes, params)

        """
        args = {"volumes" : volumes,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Part/MergeVolumes"
        self._model._print_logs_before_command("merge_volumes", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("merge_volumes", MergeVolumesResults(model = self._model, json_data = result))
        return MergeVolumesResults(model = self._model, json_data = result)

    def delete_volumes(self, volumes : Iterable[int], params : DeleteVolumesParams) -> DeleteVolumesResults:
        """ Delete volumes by deleting its face zonelets.


        Parameters
        ----------
        volumes : Iterable[int]
            Ids of volumes to be deleted.
        params : DeleteVolumesParams
            Parameters to delete volumes.

        Returns
        -------
        DeleteVolumesResults
            Returns the DeleteVolumesResults.


        Examples
        --------
        params = prime.DeleteVolumesParams(model = model)
        results = part.delete_volumes(volumes, params)

        """
        args = {"volumes" : volumes,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Part/DeleteVolumes"
        self._model._print_logs_before_command("delete_volumes", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("delete_volumes", DeleteVolumesResults(model = self._model, json_data = result))
        return DeleteVolumesResults(model = self._model, json_data = result)

    def get_face_zonelets_of_volumes(self, volumes : Iterable[int]) -> Iterable[int]:
        """ Get the face zonelets of given volumes.


        Parameters
        ----------
        volumes : Iterable[int]
            Ids of volumes.

        Returns
        -------
        Iterable[int]
            Return the ids of face zonelets.


        Examples
        --------
        >>> face_zonelets = part.get_face_zonelets_of_volumes(volumes)

        """
        args = {"volumes" : volumes}
        command_name = "PrimeMesh::Part/GetFaceZoneletsOfVolumes"
        self._model._print_logs_before_command("get_face_zonelets_of_volumes", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_face_zonelets_of_volumes")
        return result

    def compute_closed_volumes(self, params : ComputeVolumesParams) -> ComputeVolumesResults:
        """ Computes volume by identifying closed volumes defined by face zonelets of the part.


        Parameters
        ----------
        params : ComputeVolumesParams
            Parameters to compute volumes.

        Returns
        -------
        ComputeVolumesResults
            Returns the ComputeVolumesResults.


        Examples
        --------
        >>> params = prime.ComputeVolumesParams(model = model, create_zones_type = prime.CreateVolumeZonesType.PERVOLUME)
        >>> results = part.compute_closed_volumes(params)

        """
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::Part/ComputeClosedVolumes"
        self._model._print_logs_before_command("compute_closed_volumes", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("compute_closed_volumes", ComputeVolumesResults(model = self._model, json_data = result))
        return ComputeVolumesResults(model = self._model, json_data = result)

    def extract_volumes(self, face_zonelets : Iterable[int], params : ExtractVolumesParams) -> ExtractVolumesResults:
        """ Extract volumes connected to given face zonelets.


        Parameters
        ----------
        face_zonelets : Iterable[int]
            Ids of face zonelets connected to volumes.
        params : ExtractVolumesParams
            Parameters to compute volumes.

        Returns
        -------
        ExtractVolumesResults
            Return the ExtractVolumesResults.


        Examples
        --------
        >>> results = part.extract_volumes(face_zonelets, params)

        """
        args = {"face_zonelets" : face_zonelets,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Part/ExtractVolumes"
        self._model._print_logs_before_command("extract_volumes", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("extract_volumes", ExtractVolumesResults(model = self._model, json_data = result))
        return ExtractVolumesResults(model = self._model, json_data = result)

    def compute_topo_volumes(self, params : ComputeVolumesParams) -> ComputeTopoVolumesResults:
        """ Compute topovolumes by identifying closed volumes defined by topofaces of the part.


        Parameters
        ----------
        params : ComputeVolumesParams
            Parameters to compute topovolumes.

        Returns
        -------
        ComputeTopoVolumesResults
            Return the ComputeTopoVolumesResults.


        Examples
        --------
        >>> params = prime.ComputeVolumesParams(model = model, create_zones_type = prime.CreateVolumeZonesType.PERVOLUME)
        >>> results = part.compute_topo_volumes(params)

        """
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::Part/ComputeTopoVolumes"
        self._model._print_logs_before_command("compute_topo_volumes", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("compute_topo_volumes", ComputeTopoVolumesResults(model = self._model, json_data = result))
        return ComputeTopoVolumesResults(model = self._model, json_data = result)

    def extract_topo_volumes(self, topo_faces : Iterable[int], params : ExtractTopoVolumesParams) -> ExtractTopoVolumesResults:
        """ Extract topovolumes connected to given cap topofaces.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of topofaces connected to topovolumes.
        params : ExtractTopoVolumesParams
            Parameters to compute topovolumes.

        Returns
        -------
        ExtractTopoVolumesResults
            Return the ExtractTopoVolumesResults.


        Examples
        --------
        >>> results = part.extract_flow_topo_volumes(topo_faces, params)

        """
        args = {"topo_faces" : topo_faces,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::Part/ExtractTopoVolumes"
        self._model._print_logs_before_command("extract_topo_volumes", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("extract_topo_volumes", ExtractTopoVolumesResults(model = self._model, json_data = result))
        return ExtractTopoVolumesResults(model = self._model, json_data = result)

    def get_volumes_of_face_zonelet(self, face_zonelet : int) -> Iterable[int]:
        """ Get volume ids of given face zonelet.


        Parameters
        ----------
        face_zonelet : int
            Id of face zonelet.

        Returns
        -------
        Iterable[int]
            Return volume ids of given face zonelet.


        Examples
        --------
        >>> volumes = part.get_volumes_of_face_zonelet(face_zonelet)

        """
        args = {"face_zonelet" : face_zonelet}
        command_name = "PrimeMesh::Part/GetVolumesOfFaceZonelet"
        self._model._print_logs_before_command("get_volumes_of_face_zonelet", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_volumes_of_face_zonelet")
        return result

    def get_volumes(self) -> Iterable[int]:
        """ Get all the volumes of the part.


        Returns
        -------
        Iterable[int]
            Return ids of volumes.


        Examples
        --------
        >>> volumes = part.get_volumes()

        """
        args = {}
        command_name = "PrimeMesh::Part/GetVolumes"
        self._model._print_logs_before_command("get_volumes", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_volumes")
        return result

    def delete_zonelets(self, zonelets : Iterable[int]) -> DeleteResults:
        """ Delete given face zonelets.


        Parameters
        ----------
        zonelets : Iterable[int]
            Ids of zonelets to be deleted.

        Returns
        -------
        DeleteResults
            Return DeleteResults.


        Examples
        --------
        >>> results = part.delete_zonelets(zonelets)

        """
        args = {"zonelets" : zonelets}
        command_name = "PrimeMesh::Part/DeleteZonelets"
        self._model._print_logs_before_command("delete_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("delete_zonelets", DeleteResults(model = self._model, json_data = result))
        return DeleteResults(model = self._model, json_data = result)

    def get_topo_faces(self) -> Iterable[int]:
        """ Get the topofaces of a part.


        Returns
        -------
        Iterable[int]
            Return the ids of topofaces.

Return the ids of topofaces.


        Examples
        --------
        >>> topo_faces = part.get_topo_faces()

        """
        args = {}
        command_name = "PrimeMesh::Part/GetTopoFaces"
        self._model._print_logs_before_command("get_topo_faces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_topo_faces")
        return result

    def add_topo_entities_to_zone(self, zone_id : int, topo_entities : Iterable[int]) -> AddToZoneResults:
        """ Add topoentities to zone.


        Parameters
        ----------
        zone_id : int
            Id of a zone .
        topo_entities : Iterable[int]
            Ids of topoentities to be added.

        Returns
        -------
        AddToZoneResults
            Returns the AddToZoneResults.


        Examples
        --------
        >>> results = part.add_topo_entities_to_zone(zone_id, topo_entities)

        """
        args = {"zone_id" : zone_id,
        "topo_entities" : topo_entities}
        command_name = "PrimeMesh::Part/AddTopoEntitiesToZone"
        self._model._print_logs_before_command("add_topo_entities_to_zone", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("add_topo_entities_to_zone", AddToZoneResults(model = self._model, json_data = result))
        return AddToZoneResults(model = self._model, json_data = result)

    def add_zonelets_to_zone(self, zone_id : int, zonelets : Iterable[int]) -> AddToZoneResults:
        """ Add zonelets to zone.


        Parameters
        ----------
        zone_id : int
            Id of a zone .
        zonelets : Iterable[int]
            Ids of zonelets to be added.

        Returns
        -------
        AddToZoneResults
            Returns the AddToZoneResults.


        Examples
        --------
        >>> results = part.add_zonelets_to_zone(zone_id, zonelets)

        """
        args = {"zone_id" : zone_id,
        "zonelets" : zonelets}
        command_name = "PrimeMesh::Part/AddZoneletsToZone"
        self._model._print_logs_before_command("add_zonelets_to_zone", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("add_zonelets_to_zone", AddToZoneResults(model = self._model, json_data = result))
        return AddToZoneResults(model = self._model, json_data = result)

    def add_volumes_to_zone(self, zone_id : int, volumes : Iterable[int]) -> AddToZoneResults:
        """ Add volumes to zone.


        Parameters
        ----------
        zone_id : int
            Id of a zone .
        volumes : Iterable[int]
            Ids of volumes to be added.

        Returns
        -------
        AddToZoneResults
            Returns the AddToZoneResults.


        Examples
        --------
        >>> results = part.add_volumes_to_zone(zone_id, volumes)

        """
        args = {"zone_id" : zone_id,
        "volumes" : volumes}
        command_name = "PrimeMesh::Part/AddVolumesToZone"
        self._model._print_logs_before_command("add_volumes_to_zone", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("add_volumes_to_zone", AddToZoneResults(model = self._model, json_data = result))
        return AddToZoneResults(model = self._model, json_data = result)

    def get_volume_zone_of_volume(self, volume : int) -> int:
        """ Gets the volume zone of given volume.


        Parameters
        ----------
        volume : int
            Id of volume.

        Returns
        -------
        int
            Returns the id of volume zone.


        Examples
        --------
        >>> volume_zone = part.get_volume_zone_of_volume(volume)

        """
        args = {"volume" : volume}
        command_name = "PrimeMesh::Part/GetVolumeZoneOfVolume"
        self._model._print_logs_before_command("get_volume_zone_of_volume", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_volume_zone_of_volume")
        return result

    def get_face_zone_of_zonelet(self, zonelet : int) -> int:
        """ Gets the face zone of given zonelet.


        Parameters
        ----------
        zonelet : int
            Id of zonelet.

        Returns
        -------
        int
            Returns the id of face zone.


        Examples
        --------
        >>> face_zone = part.get_face_zone_of_zonelet(zonelet)

        """
        args = {"zonelet" : zonelet}
        command_name = "PrimeMesh::Part/GetFaceZoneOfZonelet"
        self._model._print_logs_before_command("get_face_zone_of_zonelet", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_face_zone_of_zonelet")
        return result

    def get_face_zones(self) -> Iterable[int]:
        """ Get all the face zones of the part.


        Returns
        -------
        Iterable[int]
            Return ids of face zones.


        Examples
        --------
        >>> face_zones = part.get_face_zones()

        """
        args = {}
        command_name = "PrimeMesh::Part/GetFaceZones"
        self._model._print_logs_before_command("get_face_zones", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_face_zones")
        return result

    def get_volume_zones(self) -> Iterable[int]:
        """ Get all the volume zones of the part.


        Returns
        -------
        Iterable[int]
            Return ids of volume zones.


        Examples
        --------
        >>> volume_zones = part.get_volume_zones()

        """
        args = {}
        command_name = "PrimeMesh::Part/GetVolumeZones"
        self._model._print_logs_before_command("get_volume_zones", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_volume_zones")
        return result

    def remove_zone_on_volumes(self, volumes : Iterable[int]) -> RemoveZoneResults:
        """ Removes zone on the given volumes.


        Parameters
        ----------
        volumes : Iterable[int]
            Volume ids whose zone is to be removed.

        Returns
        -------
        RemoveZoneResults
            Returns the RemoveZoneResults.


        Examples
        --------
        >>> part.remove_zone_on_volumes(volumes)

        """
        args = {"volumes" : volumes}
        command_name = "PrimeMesh::Part/RemoveZoneOnVolumes"
        self._model._print_logs_before_command("remove_zone_on_volumes", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("remove_zone_on_volumes", RemoveZoneResults(model = self._model, json_data = result))
        return RemoveZoneResults(model = self._model, json_data = result)

    def remove_zone_on_zonelets(self, zonelets : Iterable[int]) -> RemoveZoneResults:
        """ Removes zone on the given zonelets.


        Parameters
        ----------
        zonelets : Iterable[int]
            Zonelet ids whose zone is to be removed.

        Returns
        -------
        RemoveZoneResults
            Returns the RemoveZoneResults.


        Examples
        --------
        >>> part.remove_zone_on_zonelets(zonelets)

        """
        args = {"zonelets" : zonelets}
        command_name = "PrimeMesh::Part/RemoveZoneOnZonelets"
        self._model._print_logs_before_command("remove_zone_on_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("remove_zone_on_zonelets", RemoveZoneResults(model = self._model, json_data = result))
        return RemoveZoneResults(model = self._model, json_data = result)

    def remove_zone_on_topo_entities(self, topo_entities : Iterable[int]) -> RemoveZoneResults:
        """ Removes zone on the given topoentities.


        Parameters
        ----------
        topo_entities : Iterable[int]
            Topoentity ids whose zone is to be removed.

        Returns
        -------
        RemoveZoneResults
            Returns the RemoveZoneResults.


        Examples
        --------
        >>> part.remove_zone_on_topo_entities(topo_entities)

        """
        args = {"topo_entities" : topo_entities}
        command_name = "PrimeMesh::Part/RemoveZoneOnTopoEntities"
        self._model._print_logs_before_command("remove_zone_on_topo_entities", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("remove_zone_on_topo_entities", RemoveZoneResults(model = self._model, json_data = result))
        return RemoveZoneResults(model = self._model, json_data = result)

    def get_labels(self) -> List[str]:
        """ Get all labels on entities of part.


        Returns
        -------
        List[str]
            Return labels on entities of part.


        Examples
        --------
        >>> part.get_labels()

        """
        args = {}
        command_name = "PrimeMesh::Part/GetLabels"
        self._model._print_logs_before_command("get_labels", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_labels")
        return result

    def delete_topo_entities(self, params : DeleteTopoEntitiesParams) -> DeleteTopoEntitiesResults:
        """ Delete topoentities of part controled by parameters.


        Parameters
        ----------
        params : DeleteTopoEntitiesParams
            Parameters for control delete topoentities operation.

        Returns
        -------
        DeleteTopoEntitiesResults
            Return results of delete topoentities.


        Examples
        --------
        >>> results = part.delete_topo_entities(params)

        """
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::Part/DeleteTopoEntities"
        self._model._print_logs_before_command("delete_topo_entities", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("delete_topo_entities", DeleteTopoEntitiesResults(model = self._model, json_data = result))
        return DeleteTopoEntitiesResults(model = self._model, json_data = result)

    def get_splines(self) -> Iterable[int]:
        """ Gets the list of spline ids.


        Returns
        -------
        Iterable[int]
            Returns the list of spline ids.


        Examples
        --------
        >>> from ansys.meshing.prime import Part
        >>> results = part.get_splines()

        """
        args = {}
        command_name = "PrimeMesh::Part/GetSplines"
        self._model._print_logs_before_command("get_splines", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_splines")
        return result

    def get_summary(self, params : PartSummaryParams) -> PartSummaryResults:
        """ Get the part summary.

        Provides the part summary for the given parameters.

        Parameters
        ----------
        params : PartSummaryParams
            Part summary parameters.

        Returns
        -------
        PartSummaryResults
            Return the PartSummaryResults.

        Examples
        --------
        >>> results = part.get_summary(PartSummaryParams(model=model))

        """
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::Part/GetSummary"
        self._model._print_logs_before_command("get_summary", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_summary", PartSummaryResults(model = self._model, json_data = result))
        return PartSummaryResults(model = self._model, json_data = result)

    @property
    def id(self):
        """ Get the id of Part."""
        return self._id

    @property
    def name(self):
        """ Get the name of Part."""
        return self._name



""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class PeriodicControlParams(CoreObject):
    """PeriodicControlParams contains the parameters for periodic surface recovery.  Parameters like center, axis and angle determine the transformation from  one surface to its matching periodic surface.
    """
    _default_params = {}

    def __initialize(
            self,
            center: Iterable[float],
            axis: Iterable[float],
            angle: float):
        self._center = center if isinstance(center, np.ndarray) else np.array(center, dtype=np.double) if center is not None else None
        self._axis = axis if isinstance(axis, np.ndarray) else np.array(axis, dtype=np.double) if axis is not None else None
        self._angle = angle

    def __init__(
            self,
            model: CommunicationManager=None,
            center: Iterable[float] = None,
            axis: Iterable[float] = None,
            angle: float = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the PeriodicControlParams.

        Parameters
        ----------
        model: Model
            Model to create a PeriodicControlParams object with default parameters.
        center: Iterable[float], optional
            Center coordinates.
        axis: Iterable[float], optional
            Axis vector coordinates.
        angle: float, optional
            Angle in degrees.
        json_data: dict, optional
            JSON dictionary to create a PeriodicControlParams object with provided parameters.

        Examples
        --------
        >>> periodic_control_params = prime.PeriodicControlParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["center"] if "center" in json_data else None,
                json_data["axis"] if "axis" in json_data else None,
                json_data["angle"] if "angle" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [center, axis, angle])
            if all_field_specified:
                self.__initialize(
                    center,
                    axis,
                    angle)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "PeriodicControlParams")
                    json_data = param_json["PeriodicControlParams"] if "PeriodicControlParams" in param_json else {}
                    self.__initialize(
                        center if center is not None else ( PeriodicControlParams._default_params["center"] if "center" in PeriodicControlParams._default_params else (json_data["center"] if "center" in json_data else None)),
                        axis if axis is not None else ( PeriodicControlParams._default_params["axis"] if "axis" in PeriodicControlParams._default_params else (json_data["axis"] if "axis" in json_data else None)),
                        angle if angle is not None else ( PeriodicControlParams._default_params["angle"] if "angle" in PeriodicControlParams._default_params else (json_data["angle"] if "angle" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            center: Iterable[float] = None,
            axis: Iterable[float] = None,
            angle: float = None):
        """Set the default values of PeriodicControlParams.

        Parameters
        ----------
        center: Iterable[float], optional
            Center coordinates.
        axis: Iterable[float], optional
            Axis vector coordinates.
        angle: float, optional
            Angle in degrees.
        """
        args = locals()
        [PeriodicControlParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of PeriodicControlParams.

        Examples
        --------
        >>> PeriodicControlParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in PeriodicControlParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._center is not None:
            json_data["center"] = self._center
        if self._axis is not None:
            json_data["axis"] = self._axis
        if self._angle is not None:
            json_data["angle"] = self._angle
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "center :  %s\naxis :  %s\nangle :  %s" % (self._center, self._axis, self._angle)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def center(self) -> Iterable[float]:
        """Center coordinates.
        """
        return self._center

    @center.setter
    def center(self, value: Iterable[float]):
        self._center = value

    @property
    def axis(self) -> Iterable[float]:
        """Axis vector coordinates.
        """
        return self._axis

    @axis.setter
    def axis(self, value: Iterable[float]):
        self._axis = value

    @property
    def angle(self) -> float:
        """Angle in degrees.
        """
        return self._angle

    @angle.setter
    def angle(self, value: float):
        self._angle = value

class PeriodicControlSummaryResult(CoreObject):
    """Results of Periodic control summary.
    """
    _default_params = {}

    def __initialize(
            self,
            message: str):
        self._message = message

    def __init__(
            self,
            model: CommunicationManager=None,
            message: str = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the PeriodicControlSummaryResult.

        Parameters
        ----------
        model: Model
            Model to create a PeriodicControlSummaryResult object with default parameters.
        message: str, optional
            String with the periodic control summary.
        json_data: dict, optional
            JSON dictionary to create a PeriodicControlSummaryResult object with provided parameters.

        Examples
        --------
        >>> periodic_control_summary_result = prime.PeriodicControlSummaryResult(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["message"] if "message" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [message])
            if all_field_specified:
                self.__initialize(
                    message)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "PeriodicControlSummaryResult")
                    json_data = param_json["PeriodicControlSummaryResult"] if "PeriodicControlSummaryResult" in param_json else {}
                    self.__initialize(
                        message if message is not None else ( PeriodicControlSummaryResult._default_params["message"] if "message" in PeriodicControlSummaryResult._default_params else (json_data["message"] if "message" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            message: str = None):
        """Set the default values of PeriodicControlSummaryResult.

        Parameters
        ----------
        message: str, optional
            String with the periodic control summary.
        """
        args = locals()
        [PeriodicControlSummaryResult._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of PeriodicControlSummaryResult.

        Examples
        --------
        >>> PeriodicControlSummaryResult.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in PeriodicControlSummaryResult._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._message is not None:
            json_data["message"] = self._message
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "message :  %s" % (self._message)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def message(self) -> str:
        """String with the periodic control summary.
        """
        return self._message

    @message.setter
    def message(self, value: str):
        self._message = value

class PeriodicControlSummaryParams(CoreObject):
    """Parameters used to get size control summary.
    """
    _default_params = {}

    def __initialize(
            self):
        pass

    def __init__(
            self,
            model: CommunicationManager=None,
            json_data : dict = None,
             **kwargs):
        """Initializes the PeriodicControlSummaryParams.

        Parameters
        ----------
        model: Model
            Model to create a PeriodicControlSummaryParams object with default parameters.
        json_data: dict, optional
            JSON dictionary to create a PeriodicControlSummaryParams object with provided parameters.

        Examples
        --------
        >>> periodic_control_summary_params = prime.PeriodicControlSummaryParams(model = model)
        """
        if json_data:
            self.__initialize()
        else:
            all_field_specified = all(arg is not None for arg in [])
            if all_field_specified:
                self.__initialize()
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "PeriodicControlSummaryParams")
                    json_data = param_json["PeriodicControlSummaryParams"] if "PeriodicControlSummaryParams" in param_json else {}
                    self.__initialize()
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default():
        """Set the default values of PeriodicControlSummaryParams.

        """
        args = locals()
        [PeriodicControlSummaryParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of PeriodicControlSummaryParams.

        Examples
        --------
        >>> PeriodicControlSummaryParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in PeriodicControlSummaryParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "" % ()
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message



import PrimePyAnsysPrimeServer as Prime

import ansys.meshing.prime.internals.config as config
import ansys.meshing.prime.internals.json_utils as json
from ansys.meshing.prime.internals.communicator import Communicator
from ansys.meshing.prime.internals.error_handling import (
    communicator_error_handler,
    error_code_handler,
)

global return_value
return_value = ""


class PrimeCommunicator(Communicator):
    def __init__(self):
        Prime.SetupForPyPrime_Beta(1)

    @error_code_handler
    @communicator_error_handler
    def serve(self, model, command, *args, **kwargs) -> dict:
        command = {"Command": command}
        if len(args) > 0:
            command.update({"ObjectID": args[0]})
        if kwargs is not None:
            if "args" in kwargs:
                command.update({"Args": kwargs["args"]})

        if config.is_optimizing_numpy_arrays():
            return json.loads(
                Prime.ServeJsonBinary(model._object_id, json.dumps(command)).AsBytes()
            )

        return json.loads(Prime.ServeJson(model._object_id, json.dumps(command)).Get())

    def initialize_params(self, model, param_name: str) -> dict:
        command = {
            "ParamName": param_name,
        }

        with config.numpy_array_optimization_disabled():
            res = json.loads(Prime.GetParamDefaultJson(model._object_id, json.dumps(command)).Get())

        return res

    def run_on_server(self, model, recipe: str) -> dict:
        exec(recipe, globals())
        output = '{"Results" : "' + str(return_value) + '"}'
        with config.numpy_array_optimization_disabled():
            result = json.loads(output)
        return result

    def close(self):
        pass



from typing import List

# isort: split
from ansys.meshing.prime.autogen.fileio import FileIO as _FileIO

# isort: split
import ansys.meshing.prime.internals.utils as utils
from ansys.meshing.prime.autogen.fileiostructs import (
    ExportBoundaryFittedSplineParams,
    ExportFluentCaseParams,
    ExportFluentMeshingMeshParams,
    ExportMapdlCdbParams,
    ExportMapdlCdbResults,
    FileReadParams,
    FileReadResults,
    FileWriteParams,
    FileWriteResults,
    ImportCadParams,
    ImportCadResults,
    ImportFluentCaseParams,
    ImportFluentCaseResults,
    ImportFluentMeshingMeshParams,
    ImportFluentMeshingMeshResults,
    ImportMapdlCdbParams,
    ImportMapdlCdbResults,
    ReadSizeFieldParams,
    SizeFieldFileReadResults,
    WriteSizeFieldParams,
)
from ansys.meshing.prime.core.model import Model
from ansys.meshing.prime.params.primestructs import ErrorCode


class FileIO(_FileIO):
    __doc__ = _FileIO.__doc__

    def __init__(self, model: Model):
        """__init__(FileIO self, Model model)"""
        self._model = model
        _FileIO.__init__(self, model)

    def read_pmdat(self, file_name: str, file_read_params: FileReadParams) -> FileReadResults:
        """Function that reads PyPrimeMesh data file.

        PyPrimeMesh data files have pmdat extension.

        Parameters
        ----------
        file_name : str
            Path to file on disk.
        file_read_params : FileReadParams
            Parameter to read a file.

        Returns
        -------
        FileReadResults
            Returns FileReadResults.

        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> import ansys.meshing.prime as prime
        >>> #connect client to server and get model from it
        >>> client = prime.Client(ip="localhost", port=50060)
        >>> model = client.model
        >>> file_io = prime.FileIO(model=model)
        >>> file_read_params = prime.FileReadParams(model=model)
        >>> results = file_io.read_pmdat("/tmp/file.pmdat", file_read_params)

        """
        with utils.file_read_context(self._model, file_name) as temp_file_name:
            result = _FileIO.read_pmdat(self, temp_file_name, file_read_params)
            if result.error_code == ErrorCode.NOERROR:
                self._model._sync_up_model()
        return result

    def write_pmdat(self, file_name: str, file_write_params: FileWriteParams) -> FileWriteResults:
        """Writes PyPrimeMesh data file. PyPrimeMesh data files have .pmdat extension.


        Parameters
        ----------
        file_name : str
            Path to write file on disk.
        file_write_params : FileWriteParams
            Parameters to write PyPrimeMesh data file.

        Returns
        -------
        FileWriteResults
            Returns the FileWriteResults structure.

        Examples
        --------
        >>> results = file_io.write_pmdat("/tmp/prime_mesh_data.pmdat",
                                          prime.FileWriteParams(model=model))
        """
        with utils.file_write_context(self._model, file_name) as temp_file_name:
            result = super().write_pmdat(temp_file_name, file_write_params)
        return result

    def import_fluent_meshing_size_field(self, file_name: str) -> SizeFieldFileReadResults:
        """Import Fluent Meshing sizefield file from disk.

        Fluent Meshing sizefield files have sf and sf.gz extension.

        Parameters
        ----------
        file_name : str
            Path to file on disk

        Returns
        -------
        SizeFieldFileReadResults
            Return the SizeFieldFileReadResults.

        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> file_io = prime.FileIO(model=model)
        >>> results = file_io.import_fluent_meshing_size_field("/tmp/my_sizefield.sf")
        """
        with utils.file_read_context(self._model, file_name) as temp_file_name:
            result = super().import_fluent_meshing_size_field(temp_file_name)
        return result

    def read_size_field(
        self, file_name: str, params: ReadSizeFieldParams
    ) -> SizeFieldFileReadResults:
        """Read PyPrimeMesh sizefield file from disk.

        PyPrimeMesh sizefield files have psf and psf.gz extension.

        Parameters
        ----------
        file_name : str
            Path to file on disk.
        params : ReadSizeFieldParams
            Parameters to read size field file.

        Returns
        -------
        SizeFieldFileReadResults
            Return the SizeFieldFileReadResults.

        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> file_io = prime.FileIO(model=model)
        >>> params = prime.ReadSizeFieldParams(model=model)
        >>> results = file_io.read_size_field("/tmp/my_prime_sizefield.psf", params)
        """
        with utils.file_read_context(self._model, file_name) as temp_file_name:
            result = super().read_size_field(temp_file_name, params)
        return result

    def write_size_field(self, file_name: str, params: WriteSizeFieldParams) -> FileWriteResults:
        """Write PyPrimeMesh sizefield (.psf) to file.

        Parameters
        ----------
        file_name : str
            Path to file on disk.
        params : WriteSizeFieldParams
            Parameters to write size field file.

        Returns
        -------
        FileWriteResults
            Return the FileWriteResults.

        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> file_io = prime.FileIO(model=model)
        >>> params = prime.WriteSizeFieldParams(model=model)
        >>> results = file_io.write_size_field("/tmp/my_prime_sizefield.psf", params)
        """
        with utils.file_write_context(self._model, file_name) as temp_file_name:
            result = super().write_size_field(temp_file_name, params)
        return result

    def import_cad(self, file_name: str, params: ImportCadParams) -> ImportCadResults:
        """Function that imports CAD file.

        Import CAD file from disk.

        Supported formats on Windows are:

        \*.scdoc \*.fmd \*.agdb \*.pmdb \*.meshdat \*.mechdat \*.dsdb \*.cmdb \*.sat \*.sab
        \*.dwg \*.dxf \*.model \*.exp \*.CATPart \*.CATProduct \*.cgr \*.3dxml \*.prt\* \*.asm\*
        \*.iges \*.igs \*.ipt \*.iam \*.jt \*.prt \*.x_t \*.x_b \*.par \*.psm \*.asm \*.sldprt
        \*.sldasm \*.step \*.stp \*.stl \*.plmxml \*.tgf

        Supported formats on Linux are:

        \*.fmd \*.agdb \*.pmdb \*.meshdat \*.mechdat \*.dsdb \*.cmdb \*.sat \*.sab
        \*.CATPart \*.CATProduct \*.iges \*.igs \*.jt \*.x_t \*.x_b \*.step \*.stp
        \*.stl \*.plmxml \*.tgf

        Refer documentation for detailed list of supported formats.

        Parameters
        ----------
        file_name : str
             Path to file on disk.

        params : ImportCadParams
             Parameters to control CAD import options

        Returns
        -------
        ImportCadResults
             Returns ImportCadResults.

        Examples
        --------
        >>> import ansys.meshing.prime as prime
        >>> #connect client to server and get model from it
        >>> client = prime.Client(ip="localhost", port=50060)
        >>> model = client.model
        >>> file_io = prime.FileIO(model=model)
        >>> params = ImportCadParams(model=model)
        >>> results = file_io.import_cad(
                        "/tmp/my_cad.x_t", params=params)

        """
        with utils.file_read_context(self._model, file_name) as temp_file_name:
            import_result = _FileIO.import_cad(self, temp_file_name, params)
            if import_result.error_code == ErrorCode.NOERROR:
                self._model._sync_up_model()
        return import_result

    def import_fluent_meshing_meshes(
        self,
        file_names: List[str],
        import_fluent_meshing_mesh_params: ImportFluentMeshingMeshParams,
    ) -> ImportFluentMeshingMeshResults:
        """Imports Fluent Meshing meshes of given files on disk.

        Fluent Meshing mesh files have msh and msh.gz extension.

        Parameters
        ----------
        file_names : List[str]
            Full path of files to be imported.
        import_fluent_meshing_mesh_params : ImportFluentMeshingMeshParams
            Parameters to import Fluent Meshing mesh.

        Returns
        -------
        ImportFluentMeshingMeshResults
            Returns the FileReadResults.


        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> file_io = prime.FileIO(model=model)
        >>> params = prime.ImportFluentMeshingMeshParams(model=model)
        >>> results = file_io.import_fluent_meshing_meshes(
                        ["/tmp/mesh.msh", "/tmp/mesh1.msh"],
                        params)

        """
        with utils.file_read_context_list(self._model, file_names) as temp_file_names:
            result = _FileIO.import_fluent_meshing_meshes(
                self, temp_file_names, import_fluent_meshing_mesh_params
            )
        if result.error_code == ErrorCode.NOERROR:
            self._model._sync_up_model()
        return result

    def import_fluent_case(
        self, file_name: str, import_fluent_case_params: ImportFluentCaseParams
    ) -> ImportFluentCaseResults:
        """Imports Fluent case file on disk.

        Fluent case files have cas extension.

        Parameters
        ----------
        file_name : str
            Path to file on disk.
        import_fluent_case_params : ImportFluentCaseParams
            Parameters to import fluent case file.

        Returns
        -------
        ImportFluentCaseResults
            Returns the ImportFluentCaseResults.


        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> file_io = prime.FileIO(model=model)
        >>> params = prime.ImportFluentCaseParams(model=model)
        >>> results = file_io.import_fluent_case("/tmp/fluent.cas", params)

        """
        with utils.file_read_context(self._model, file_name) as temp_file_name:
            result = _FileIO.import_fluent_case(self, temp_file_name, import_fluent_case_params)
            if result.error_code == ErrorCode.NOERROR:
                self._model._sync_up_model()
        return result

    def export_fluent_case(
        self, file_name: str, export_fluent_case_params: ExportFluentCaseParams
    ) -> FileWriteResults:
        """Exports Fluent case file. Fluent case files have cas extension.

        Parameters
        ----------
        file_name : str
            Path to file on disk.
        export_fluent_case_params : ExportFluentCaseParams
            Parameters to export fluent case file.

        Returns
        -------
        FileWriteResults
            Returns the FileWriteResults structure.

        Examples
        --------
        >>> file_io = FileIO(model=model)
        >>> results = file_io.export_fluent_case(
                        "/tmp/fluent.cas",
                        prime.ExportFluentCaseParams(model=model))
        """
        with utils.file_write_context(self._model, file_name) as temp_file_name:
            result = super().export_fluent_case(temp_file_name, export_fluent_case_params)
        return result

    def import_mapdl_cdb(
        self, file_name: str, params: ImportMapdlCdbParams
    ) -> ImportMapdlCdbResults:
        """Function that imports MAPDL CDB file(cdb).

        Parameters
        ----------
        file_name : str
            Path to file on disk.
        params : ImportMapdlCdbParams
            Parameter to import a CDB file.

        Returns
        -------
        ImportMapdlCdbResults
            Returns ImportMapdlCdbResults.

        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> import ansys.meshing.prime as prime
        >>> #connect client to server and get model from it
        >>> client = prime.Client(ip="localhost", port=50060)
        >>> model = client.model
        >>> file_io = prime.FileIO(model=model)
        >>> params = prime.ImportMapdlCdbParams(model=model)
        >>> results = file_io.import_mapdl_cdb("/tmp/file.cdb", params)
        """
        with utils.file_read_context(self._model, file_name) as temp_file_name:
            result = _FileIO.import_mapdl_cdb(self, temp_file_name, params)
            if result.error_code == ErrorCode.NOERROR:
                self._model._sync_up_model()
        return result

    def export_mapdl_cdb(
        self, file_name: str, params: ExportMapdlCdbParams
    ) -> ExportMapdlCdbResults:
        """Function that exports MAPDL CDB file(cdb).

        Parameters
        ----------
        file_name : str
            Path to file on disk.
        params : ExportMapdlCdbParams
            Parameter to export a CDB file.

        Returns
        -------
        ExportMapdlCdbResults
            Returns ExportMapdlCdbResults.

        Notes
        -----
        This API does not support Unicode paths now.

        Examples
        --------
        >>> import ansys.meshing.prime as prime
        >>> #connect client to server and get model from it
        >>> client = prime.Client(ip="localhost", port=50060)
        >>> model = client.model
        >>> file_io = prime.FileIO(model=model)
        >>> params = prime.ExportMapdlCdbParams(model=model)
        >>> results = file_io.export_mapdl_cdb("/tmp/file.cdb", params)
        """
        with utils.file_write_context(self._model, file_name) as temp_file_name:
            result = super().export_mapdl_cdb(temp_file_name, params)
        return result

    def export_fluent_case(
        self, file_name: str, export_fluent_case_params: ExportFluentCaseParams
    ) -> FileWriteResults:
        """Exports Fluent case file. Fluent case files have cas extension.

        Parameters
        ----------
        file_name : str
            Path to file on disk.
        export_fluent_case_params : ExportFluentCaseParams
            Parameters to export fluent case file.

        Returns
        -------
        FileWriteResults
            Returns the FileWriteResults structure.

        Examples
        --------
        >>> file_io = FileIO(model=model)
        >>> results = file_io.export_fluent_case("/tmp/fluent.cas",
                                                 prime.ExportFluentCaseParams(model=model))
        """
        with utils.file_write_context(self._model, file_name) as temp_file_name:
            result = super().export_fluent_case(temp_file_name, export_fluent_case_params)
        return result

    def export_fluent_meshing_mesh(
        self, file_name: str, export_fluent_mesh_params: ExportFluentMeshingMeshParams
    ) -> FileWriteResults:
        """Export Fluent Meshing mesh file. Fluent Meshing mesh files have .msh extension.

        Parameters
        ----------
        file_name : str
            Path to file on disk.
        export_fluent_mesh_params : ExportFluentMeshingMeshParams
            Parameters to export Fluent Meshing mesh file.

        Returns
        -------
        FileWriteResults
            Returns the FileWriteResults structure.

        Examples
        --------
        >>> results = file_io.export_fluent_meshing_mesh(
                        "/tmp/fluent_meshing.msh",
                        ExportFluentMeshingMeshParams(model=model))
        """
        with utils.file_write_context(self._model, file_name) as temp_file_name:
            result = super().export_fluent_meshing_mesh(temp_file_name, export_fluent_mesh_params)
        return result

    def export_boundary_fitted_spline_kfile(
        self, file_name: str, export_params: ExportBoundaryFittedSplineParams
    ) -> FileWriteResults:
        """Export IGA LS-DYNA Keyword file for boundary fitted spline.

        Parameters
        ----------
        file_name : str
            Name of the file.
        export_params : ExportBoundaryFittedSplineParams
            Parameters for IGA LS-DYNA Keyword file export.

        Returns
        -------
        FileWriteResults
            Returns FileWriteResults.

        Examples
        --------
        >>> results = file_io.export_boundary_fitted_spline_k_file(
                        file_name,
                        ExportBoundaryFittedSplineParams(model=model))
        """
        with utils.file_write_context(self._model, file_name) as temp_file_name:
            result = super().export_boundary_fitted_spline_kfile(temp_file_name, export_params)
        return result



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class SizeControl(CoreObject):
    """Size control is used to compute the size field.

    The size field is computed based on the size control defined.
    Different type of size controls provide control over how the mesh size is distributed on a surface or within the volume.
    """

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """ Initialize SizeControl """
        self._model = model
        self._comm = model._communicator
        self._id = id
        self._object_id = object_id
        self._name = name
        self._freeze()

    def set_curvature_sizing_params(self, params : CurvatureSizingParams) -> SetSizingResults:
        """ Set the curvature sizing parameters to compute volumetric size field.


        Parameters
        ----------
        params : CurvatureSizingParams
            Parameters that enables you to set the normal angle as the maximum allowable angle at which one element edge may span.

        Returns
        -------
        SetSizingResults
            Return the SetSizingResults.


        Examples
        --------
        >>> size_control.set_curvature_sizing_params(
        >>>                  prime.CurvatureSizingParams(model=model,
        >>>                  min = 0.1, max = 1.0, growth_rate = 1.2,
        >>>                  normal_angle = 18))

        """
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::SizeControl/SetCurvatureSizingParams"
        self._model._print_logs_before_command("set_curvature_sizing_params", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_curvature_sizing_params", SetSizingResults(model = self._model, json_data = result))
        return SetSizingResults(model = self._model, json_data = result)

    def set_soft_sizing_params(self, params : SoftSizingParams) -> SetSizingResults:
        """ Set the soft sizing parameters to compute volumetric size field.


        Parameters
        ----------
        params : SoftSizingParams
            Parameters that enables you to set the maximum size on scoped face zonelets.

        Returns
        -------
        SetSizingResults
            Return the SetSizingResults.


        Examples
        --------
        >>> size_control.set_soft_sizing_params(
        >>>                  prime.SoftSizingParams(model=model,
        >>>                  max = 1.0, growth_rate = 1.2))

        """
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::SizeControl/SetSoftSizingParams"
        self._model._print_logs_before_command("set_soft_sizing_params", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_soft_sizing_params", SetSizingResults(model = self._model, json_data = result))
        return SetSizingResults(model = self._model, json_data = result)

    def set_proximity_sizing_params(self, params : ProximitySizingParams) -> SetSizingResults:
        """ Set the proximity sizing parameters to compute volumetric size field.


        Parameters
        ----------
        params : ProximitySizingParams
            Parameters that enables you to specify number of elements in the gaps.

        Returns
        -------
        SetSizingResults
            Return the SetSizingResults.


        Examples
        --------
        >>> size_control.set_proximity_sizing_params(
        >>>                  prime.ProximitySizingParams(model=model,
        >>>                  min = 0.1, max = 1.0, growth_rate = 1.2))

        """
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::SizeControl/SetProximitySizingParams"
        self._model._print_logs_before_command("set_proximity_sizing_params", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_proximity_sizing_params", SetSizingResults(model = self._model, json_data = result))
        return SetSizingResults(model = self._model, json_data = result)

    def set_hard_sizing_params(self, params : HardSizingParams) -> SetSizingResults:
        """ Set the hard sizing parameters to compute volumetric size field.


        Parameters
        ----------
        params : HardSizingParams
            Parameters that enables you to set uniform size based on the specified size.

        Returns
        -------
        SetSizingResults
            Return the SetSizingResults.


        Examples
        --------
        >>> size_control.set_hard_sizing_params(
        >>>                  prime.HardSizingParams(model=model,
        >>>                  min = 0.1, growth_rate = 1.2))

        """
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::SizeControl/SetHardSizingParams"
        self._model._print_logs_before_command("set_hard_sizing_params", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_hard_sizing_params", SetSizingResults(model = self._model, json_data = result))
        return SetSizingResults(model = self._model, json_data = result)

    def set_meshed_sizing_params(self, params : MeshedSizingParams) -> SetSizingResults:
        """ Set the meshed sizing parameters to compute volumetric size field.


        Parameters
        ----------
        params : MeshedSizingParams
            Parameters that enables you to set the sizes based on existing sizes.

        Returns
        -------
        SetSizingResults
            Return the SetSizingResults.


        Examples
        --------
        >>> size_control.set_meshed_sizing_params(
        >>>                  prime.MeshedSizingParams(model=model,
        >>>                  growth_rate = 1.2))

        """
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::SizeControl/SetMeshedSizingParams"
        self._model._print_logs_before_command("set_meshed_sizing_params", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_meshed_sizing_params", SetSizingResults(model = self._model, json_data = result))
        return SetSizingResults(model = self._model, json_data = result)

    def set_boi_sizing_params(self, params : BoiSizingParams) -> SetSizingResults:
        """ Set the body of influence sizing parameters to compute volumetric size field.


        Parameters
        ----------
        params : BoiSizingParams
            Parameters that enables you to set sizing on the body of influence region.

        Returns
        -------
        SetSizingResults
            Return the SetSizingResults.


        Examples
        --------
        >>> size_control.set_boi_sizing_params(
        >>>                  prime.BoiSizingParams(model=model,
        >>>                  max = 0.1, growth_rate = 1.2))

        """
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::SizeControl/SetBoiSizingParams"
        self._model._print_logs_before_command("set_boi_sizing_params", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_boi_sizing_params", SetSizingResults(model = self._model, json_data = result))
        return SetSizingResults(model = self._model, json_data = result)

    def get_curvature_sizing_params(self) -> CurvatureSizingParams:
        """ Get the curvature sizing parameters of size control.


        Returns
        -------
        CurvatureSizingParams
            Return the CurvatureSizingParams.


        Examples
        --------
        >>> params = size_control.get_curvature_sizing_params()

        """
        args = {}
        command_name = "PrimeMesh::SizeControl/GetCurvatureSizingParams"
        self._model._print_logs_before_command("get_curvature_sizing_params", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_curvature_sizing_params", CurvatureSizingParams(model = self._model, json_data = result))
        return CurvatureSizingParams(model = self._model, json_data = result)

    def get_soft_sizing_params(self) -> SoftSizingParams:
        """ Get the soft sizing parameters of size control.


        Returns
        -------
        SoftSizingParams
            Return the SoftSizingParams.


        Examples
        --------
        >>> params = size_control.get_soft_sizing_params()

        """
        args = {}
        command_name = "PrimeMesh::SizeControl/GetSoftSizingParams"
        self._model._print_logs_before_command("get_soft_sizing_params", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_soft_sizing_params", SoftSizingParams(model = self._model, json_data = result))
        return SoftSizingParams(model = self._model, json_data = result)

    def get_proximity_sizing_params(self) -> ProximitySizingParams:
        """ Get the proximity sizing parameters of size control.


        Returns
        -------
        ProximitySizingParams
            Return the ProximitySizingParams.


        Examples
        --------
        >>> params = size_control.get_proximity_sizing_params()

        """
        args = {}
        command_name = "PrimeMesh::SizeControl/GetProximitySizingParams"
        self._model._print_logs_before_command("get_proximity_sizing_params", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_proximity_sizing_params", ProximitySizingParams(model = self._model, json_data = result))
        return ProximitySizingParams(model = self._model, json_data = result)

    def get_hard_sizing_params(self) -> HardSizingParams:
        """ Get the hard sizing parameters of size control.


        Returns
        -------
        HardSizingParams
            Return the HardSizingParams.


        Examples
        --------
        >>> params = size_control.get_hard_sizing_params()

        """
        args = {}
        command_name = "PrimeMesh::SizeControl/GetHardSizingParams"
        self._model._print_logs_before_command("get_hard_sizing_params", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_hard_sizing_params", HardSizingParams(model = self._model, json_data = result))
        return HardSizingParams(model = self._model, json_data = result)

    def get_meshed_sizing_params(self) -> MeshedSizingParams:
        """ Get the meshed sizing parameters of size control.


        Returns
        -------
        MeshedSizingParams
            Return the MeshedSizingParams.


        Examples
        --------
        >>> params = size_control.get_meshed_sizing_params()

        """
        args = {}
        command_name = "PrimeMesh::SizeControl/GetMeshedSizingParams"
        self._model._print_logs_before_command("get_meshed_sizing_params", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_meshed_sizing_params", MeshedSizingParams(model = self._model, json_data = result))
        return MeshedSizingParams(model = self._model, json_data = result)

    def get_boi_sizing_params(self) -> BoiSizingParams:
        """ Get the body of influence sizing parameters of size control.


        Returns
        -------
        BoiSizingParams
            Return the BoiSizingParams.


        Examples
        --------
        >>> params = size_control.get_boi_sizing_params()

        """
        args = {}
        command_name = "PrimeMesh::SizeControl/GetBoiSizingParams"
        self._model._print_logs_before_command("get_boi_sizing_params", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_boi_sizing_params", BoiSizingParams(model = self._model, json_data = result))
        return BoiSizingParams(model = self._model, json_data = result)

    def set_suggested_name(self, name : str) -> SetNameResults:
        """ Set the unique name for the size control based on the given suggested name.


        Parameters
        ----------
        name : str
            Suggested name for the size control.

        Returns
        -------
        SetNameResults
            Return a name of the size control.


        Examples
        --------
        >>> size_control.set_suggested_name("control1")

        """
        args = {"name" : name}
        command_name = "PrimeMesh::SizeControl/SetSuggestedName"
        self._model._print_logs_before_command("set_suggested_name", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_suggested_name", SetNameResults(model = self._model, json_data = result))
        return SetNameResults(model = self._model, json_data = result)

    def set_scope(self, scope : ScopeDefinition) -> SetScopeResults:
        """ Set the scope for size control to evaluate.

        Size control uses scope to evaluate entities for which size field needs to be computed.

        Parameters
        ----------
        scope : ScopeDefinition
            ScopeDefinition to scope entities for size field computation.

        Returns
        -------
        SetScopeResults
            Return a SetScopeResults.


        Examples
        --------
        >>> size_control.set_scope(prime.ScopeDefinition(model=model,
        >>>                        entity_type = ScopeEntity.FACEZONELETS,
        >>>                        evaluation_type = ScopeEvaluationType.LABELS,
        >>>                        label_expression = "inlet"))

        """
        args = {"scope" : scope._jsonify()}
        command_name = "PrimeMesh::SizeControl/SetScope"
        self._model._print_logs_before_command("set_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_scope", SetScopeResults(model = self._model, json_data = result))
        return SetScopeResults(model = self._model, json_data = result)

    def get_scope(self) -> ScopeDefinition:
        """ Get the scope used by size control to evaluate entities.


        Returns
        -------
        ScopeDefinition
            Return the ScopeDefinition.


        Examples
        --------
        >>> scope = size_control.get_scope()

        """
        args = {}
        command_name = "PrimeMesh::SizeControl/GetScope"
        self._model._print_logs_before_command("get_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_scope", ScopeDefinition(model = self._model, json_data = result))
        return ScopeDefinition(model = self._model, json_data = result)

    def get_summary(self, params : SizeControlSummaryParams) -> SizeControlSummaryResult:
        """ Get the size control summary along with the evaluated scope for the provided parameters..


        Parameters
        ----------
        params : SizeControlSummaryParams
            Size control summary parameters.

        Returns
        -------
        SizeControlSummaryResult
            Return the SizeControlSummaryResult.

        Examples
        --------
        >>> results = size_control.get_summary(prime.SizeControlSummaryParams(model=model))

        """
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::SizeControl/GetSummary"
        self._model._print_logs_before_command("get_summary", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_summary", SizeControlSummaryResult(model = self._model, json_data = result))
        return SizeControlSummaryResult(model = self._model, json_data = result)

    @property
    def id(self):
        """ Get the id of SizeControl."""
        return self._id

    @property
    def name(self):
        """ Get the name of SizeControl."""
        return self._name



from typing import List

import numpy as np

# isort: split
from ansys.meshing.prime.autogen.wrapper import Wrapper as _Wrapper

# isort: split
from ansys.meshing.prime.autogen.connect import Connect
from ansys.meshing.prime.autogen.connectstructs import IntersectParams
from ansys.meshing.prime.autogen.controlstructs import (
    ScopeDefinition as ScopeDefinition,
)
from ansys.meshing.prime.autogen.modelstructs import MergePartsParams
from ansys.meshing.prime.autogen.partstructs import NamePatternParams
from ansys.meshing.prime.autogen.surfaceutilities import SurfaceUtilities
from ansys.meshing.prime.autogen.surfaceutilitystructs import ResolveIntersectionsParams
from ansys.meshing.prime.autogen.wrapperstructs import WrapParams as WrapParams
from ansys.meshing.prime.autogen.wrapperstructs import (
    WrapperCloseGapsParams,
    WrapperCloseGapsResult,
)
from ansys.meshing.prime.autogen.wrapperstructs import WrapResult as WrapResult
from ansys.meshing.prime.core.model import Model
from ansys.meshing.prime.core.part import Part
from ansys.meshing.prime.params.primestructs import ErrorCode as ErrorCode


class Wrapper(_Wrapper):
    """Provides operations to generate surface mesh using wrapper technology."""

    def __init__(self, model: Model):
        """__init__(Model self, int id, int object_id, char* name)"""
        _Wrapper.__init__(self, model)
        self._model = model

    def wrap(self, wrapper_control_id: int, params: WrapParams) -> WrapResult:
        """Performs wrapping with specified controls in wrapper control and with provided
            parameters.


        Parameters
        ----------
        wrapper_control_id : int
            Id of wrapper control.
        params : WrapParams
            Wrap Parameters.

        Returns
        -------
        WrapResult
            Returns the Wrap Results.


        Examples
        --------
        >>> results = wrapper.wrap(wrapper_control_id, params)

        """
        result = _Wrapper.wrap(self, wrapper_control_id, params)
        if result.error_code == ErrorCode.NOERROR:
            self._model._add_part(result.id)
        return result

    def connect(
        self,
        wrapper_part: Part,
        target_labels: List[str],
        source_parts: List[Part],
        source_labels: List[str],
    ):
        """Performs label controlled connection of wrapper part face zonelets to face zonelets of
            source parts.


        Parameters
        ----------
        wrapper_part : Part
            wrapper part.
        target_labels : List[str]
            target zonelet labels to be connected.
        source_parts : List[Part]
           source parts to be connected with.
        source_labels : List[Part]
           source zonelet labels to be connected.


        Examples
        --------
        >>> wrapper.connect(wrapper_part, target_labels, source_parts, source_labels)

        """
        for i in range(0, len(source_parts)):
            source_parts[i].add_labels_on_zonelets(
                [source_parts[i].get_name()], source_parts[i].get_face_zonelets()
            )
        name_pattern_params = NamePatternParams(self._model)
        face_zonelet_ids = [
            fz
            for label in target_labels
            for fz in wrapper_part.get_face_zonelets_of_label_name_pattern(
                label, name_pattern_params=name_pattern_params
            )
        ]
        with_face_zonelet_ids = [
            fz
            for i in range(0, len(source_parts))
            for fz in source_parts[i].get_face_zonelets_of_label_name_pattern(
                source_labels[i], name_pattern_params=name_pattern_params
            )
        ]

        part_ids = [part.id for part in source_parts]
        part_ids.append(wrapper_part.id)
        merged_part_res = self._model.merge_parts(
            part_ids=part_ids,
            params=MergePartsParams(
                model=self._model, merged_part_suggested_name=wrapper_part.name
            ),
        )

        modified_zonelets = np.array([], dtype=np.int32)
        if len(with_face_zonelet_ids) > 0 and len(face_zonelet_ids) > 0:
            # intersect
            connect = Connect(self._model)
            params = IntersectParams(model=self._model)
            res = connect.intersect_face_zonelets(
                part_id=merged_part_res.merged_part_id,
                face_zonelet_ids=face_zonelet_ids,
                with_face_zonelet_ids=with_face_zonelet_ids,
                params=params,
            )
            np.append(modified_zonelets, face_zonelet_ids)
            np.append(modified_zonelets, with_face_zonelet_ids)
        surf_utils = SurfaceUtilities(self._model)
        surf_utils.resolve_intersections(
            face_zonelet_ids=modified_zonelets, params=ResolveIntersectionsParams(model=self._model)
        )

    def close_gaps(
        self, scope: ScopeDefinition, params: WrapperCloseGapsParams
    ) -> WrapperCloseGapsResult:
        """Close gaps create patching surfaces within the face zonelets specified
            by scope using gap size.


        Parameters
        ----------
        scope : ScopeDefinition
            Scope definition of face zonelets.
        params : WrapperCloseGapsParams
            Wrapper close gaps parameters.

        Returns
        -------
        WrapperCloseGapsResult
            Returns the WrapperCloseGapsResult.


        Examples
        --------
        >>> result = wrapper.close_gaps(scope, params)

        """
        result = _Wrapper.close_gaps(self, scope, params)
        if result.error_code == ErrorCode.NOERROR:
            self._model._sync_up_model()
        return result



import os
import urllib.request
from typing import Optional
from urllib.parse import urljoin

import ansys.meshing.prime.internals.defaults as defaults

__all__ = ['download_file']


def _joinurl(base, *paths):
    for path in paths:
        if base[-1] != '/':
            base += '/'
        base = urljoin(base, path)
    return base


def _get_default_server_and_joiner():
    return 'https://github.com/pyansys/example-data/raw/master', _joinurl


def _get_filepath_on_default_server(filename: str, *directory: str):
    server, joiner = _get_default_server_and_joiner()
    if directory:
        return joiner(server, *directory, filename)
    else:
        return joiner(server, filename)


def _retrieve_url(url, dest):
    saved_file, _ = urllib.request.urlretrieve(url, filename=dest)
    return saved_file


def _retrieve_data(url: str, filename: str, dest: str = None, force: bool = False):
    if dest is None:
        dest = defaults.get_examples_path()
    local_path = os.path.join(dest, os.path.basename(filename))
    if not force and os.path.isfile(local_path):
        return local_path
    local_path = _retrieve_url(url, local_path)
    return local_path


def download_file(
    filename: str, *directory: str, destination: Optional[str] = None, force: bool = False
):
    """Download a file from PyAnsys examples Github repo

    Parameters
    ----------
    filename: str
        Name of the file to download
    destination: Optional[str]
        Optional destination to download the directory to
    force: bool
        Flag to force download even if the file exists in cache
    directory: tuple[str]
        Path under the PyAnsys Github examples repo

    Returns
    -------
    Tuple[str, str]
        Tuple containing filepath to be used and the local filepath of the downloaded directory
        The two are different in case of containers.

    """
    url = _get_filepath_on_default_server(filename, *directory)
    local_path = _retrieve_data(url, filename, dest=destination, force=force)
    return local_path



from ansys.meshing.prime.internals.communicator import Communicator


class CommunicationManager:
    def __init__(self, comm: Communicator):
        self._comm = comm

    @property
    def _communicator(self):
        return self._comm
